"""
Coin Metrics API Pro Module Definitions
"""

import logging
from .community import Community
from .errors import InvalidMetricError

class Pro(Community):
    """
    Coin Metrics API Pro Object
    """

    # Due to the conveniance methods, we trigger R0904: too-many-public-methods
    # even though this is desired.
    # pylint: disable=R0904

    def __init__(self, api_key=''):
        """
        Initialize the Pro API in much the same way as the same way as
        :py:func:`coinmetrics.base.Community.__init__`, except the
        :samp:`api_key` is now required for functionality.

        :param api_key: API key to be used for the Pro API.
        :type api_key: str, optional
        """
        super().__init__()
        self.logger = logging.getLogger(__name__)
        self.host_url = "https://api.coinmetrics.io/v2/"
        self.headers = {"api_key": api_key} if api_key != '' else {}

    def get_indexes(self):
        """
        Fetch list of available assets.

        :return: List of supported indices.
        :rtype: list
        """
        return self._api_query("indexes")['indexes']

    #: An alias for :py:func:`get_indexes`
    getindexes, indexes = [get_indexes] * 2

    def index_checker(self, indexes):
        """
        Helper function to determine if the requested index(es) is(are) valid.

        :param metric: Unique ID corresponding to index.
        :type metric: str

        :raises: InvalidMetricError
        """
        indexes = indexes.split(",")
        reference = self.get_indexes()
        for index in indexes:
            if index in reference:
                pass
            else:
                raise InvalidMetricError("Invalid metrics: '{}'".format(index))

    def get_index_info(self, indexes):
        """
        Fetch index(es) information. Including applicable assets, firendly
        name and most recent and oldest data timestamps. No specified
        asset will return all available asset info.

        :param assets: Unique ID corresponding to the index's name.
        :type assets: str, optional

        :return: Index(es) information.
        :rtype: list of dict
        """
        options = {"subset": indexes}
        return self._api_query("index_info", options)['indexesInfo']
    


"""
Coin Metrics API Base Module Definitions
"""

from decimal import Decimal
import json
import logging
import urllib.parse
import requests
from dateutil import parser
from .errors import (InvalidAssetError, InvalidTimeRangeError,
                     InvalidMetricError, InvalidExchangeError, InvalidMarketError)

class Base:
    """
    Coin Metrics API Base Object
    """
    def __init__(self, api_key=""):
        """
        Initialize API to use the Base API endpoints by default.
        An optional :samp:`api_key` can be supplied.

        :param api_key: API key to be used for the Pro API.
        :type api_key: str, optional
        """
        self.logger = logging.getLogger(__name__)
        self.host_url = 'https://community-api.coinmetrics.io/v2/'
        self.headers = {"api_key": api_key} if api_key != '' else {}

    def _api_query(self, endpoint, options=None):
        """
        Execute the raw API query and return the raw JSON output.

        :param endpoint: URL Path the query will be sent to. This includes any
                         URL based parameters.
        :type endpoint: string

        :param options: Query parameters, including asset(s), metric(s),
                        exchanges(s), and time range.
        :type options: dict, optional

        :return: Raw JSON response as dict.
        :rtype: dict
        """
        self.logger.debug("Host URL: '%s'", self.host_url)
        self.logger.debug("Endpoint: '%s'", endpoint)
        self.logger.debug("Options: '%s'", str(options))
        self.logger.debug("Headers: '%s'", str(self.headers))
        encoded_options = urllib.parse.urlencode(options if options is not None else {})
        request_url = self.host_url + endpoint + '?' + encoded_options
        self.logger.debug("Request URL: '%s'", str(request_url))
        response = requests.get(request_url, headers=self.headers).content.decode('utf-8')
        self.logger.debug("API query sent.")
        return json.loads(response, parse_float=Decimal, parse_int=Decimal)

    def get_assets(self):
        """
        Fetch list of available assets.

        :return: List of supported assets.
        :rtype: list
        """
        self.logger.debug("Fetching assets.")
        return self._api_query("assets")["assets"]

    #: An alias for :py:func:`get_assets`
    get_supported_assets, assets = [get_assets] * 2

    def asset_checker(self, assets):
        """
        Helper function to determine if the requested asset(s) is(are) valid.

        :param asset: Unique ID corresponding to the asset's ticker.
        :type asset: str

        :raises: InvalidAssetError
        """
        self.logger.debug("Checking assets: '%s'", assets)
        assets = assets.split(",")
        reference = self.get_assets()
        for asset in assets:
            if asset in reference:
                pass
            else:
                raise InvalidAssetError("Invalid asset: '{}'".format(asset))

    def get_metrics(self):
        """
        Fetch list of available metrics.

        :return: List of supported metrics.
        :rtype: list
        """
        self.logger.debug("Fetching metrics.")
        return self._api_query("metrics")['metrics']

    #: An alias for :py:func:`get_metrics`
    getmetrics, metrics = [get_metrics] * 2

    def metric_checker(self, metrics):
        """
        Helper function to determine if the requested metric(s) is(are) valid.

        :param metrics: Unique ID corresponding to metric.
        :type metrics: str

        :raises: InvalidMetricError
        """
        self.logger.debug("Checking metrics: '%s'", metrics)
        metrics = metrics.split(",")
        reference = self.get_metrics()
        for metric in metrics:
            if metric in reference:
                pass
            else:
                raise InvalidMetricError("Invalid metrics: '{}'".format(metric))

    def get_exchanges(self):
        """
        Fetch list of available exchanges.

        :return: List of supported exchanges.
        :rtype: list
        """
        self.logger.debug("Fetching exchanges.")
        return self._api_query("exchanges")['exchanges']

    #: An alias for :py:func:`get_exchanges`
    getexchanges, exchange = [get_exchanges] * 2

    def exchange_checker(self, exchanges):
        """
        Helper function to determine if the requested exchange(s) is(are) valid.

        :param exchanges: Unique ID corresponding to the exchange.
        :type exchanges: str

        :raises: InvalidExchangeError
        """
        self.logger.debug("Checking exchanges: '%s'", exchanges)
        exchanges = exchanges.split(",")
        reference = self.get_exchanges()
        for exchange in exchanges:
            if exchange in reference:
                pass
            else:
                raise InvalidExchangeError("Invalid exchange: '{}'".format(exchange))

    def get_markets(self):
        """
        Fetch list of available markets.

        :return: List of supported markets.
        :rtype: list
        """
        self.logger.debug("Fetching markets.")
        return self._api_query("markets")['markets']

    #: An alias for :py:func:`get_markets`
    getmarkets, markets = [get_markets] * 2

    def market_checker(self, markets):
        """
        Helper function to determine if the requested market(s) is(are) valid.

        :param market: Unique ID corresponding to the market.
        :type market: str

        :raises: InvalidMarketError
        """
        self.logger.debug("Checking markets: '%s'", markets)
        markets = markets.split(",")
        reference = self.get_markets()
        for market in markets:
            if market in reference:
                pass
            else:
                raise InvalidMarketError("Invalid market: '{}'".format(market))

    def timestamp_checker(self, begin_timestamp, end_timestamp):
        """
        Helper function to determine if the provided timerange is valid.

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or datetime

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or datetime

        :raises: InvalidTimeRangeError
        """
        self.logger.debug("Checking timestamps:")
        self.logger.debug("Begin Timestamp: '%s'", begin_timestamp)
        self.logger.debug("End Timestamp: '%s'", end_timestamp)
        begin_timestamp = parser.parse(str(begin_timestamp))
        end_timestamp = parser.parse(str(end_timestamp))
        if begin_timestamp <= end_timestamp:
            pass
        else:
            raise InvalidTimeRangeError(
                """Invalid time range starting: '{}', and ending: '{}'."""
                .format(begin_timestamp, end_timestamp))



"""
Coin Metrics Legacy Community API Module Definitions
"""

from decimal import Decimal
import json
import logging
import requests
from .errors import InvalidAssetError, InvalidTimeRangeError, InvalidDataTypeError

class CommunityLegacy():
    """
    Community APIv1 Object
    """

    # Due to the conveniance methods, we trigger R0904: too-many-public-methods
    # even though this is desired.
    # pylint: disable=R0904
    def __init__(self):
        """
        The Community API object definition
        """
        self.logger = logging.getLogger(__name__)
        self.host_url = 'https://coinmetrics.io/api/v1/'
        self.methods = set(['get_supported_assets',
                            'get_available_data_types_for_asset',
                            'get_asset_data_for_time_range'])

    def _api_query(self, method, *options):
        """
        Execute the raw API query and return the raw JSON output.

        :param method: Query method for getting data.
        :type method: str

        :param options: Tuple containing all options, including: asset, data_type, and timestamps.
        :type options: tuple

        :return: Raw JSON response
        :rtype: dict
        """
        self.logger.debug("Method: '%s'", method)
        self.logger.debug("Options: '%s'", str(options))
        url_path = '' if not options else '/' + '/'.join(str(option) for option in options)
        request_url = ''
        if method in self.methods:
            request_url = self.host_url + method + url_path
        else:
            print("ERROR: Not a valid method.")
        response = requests.get(request_url).content.decode('utf-8')
        return json.loads(response, parse_float=Decimal, parse_int=Decimal)

    def get_supported_assets(self):
        """
        Returns list of assets that are supported by coinmetrics.io.

        :return: List of supported assets.
        :rtype: list
        """
        return self._api_query("get_supported_assets")

    def get_available_data_types_for_asset(self, asset):
        """
        Returns list of data types available for specified asset.

        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :return: List of data types for the specified asset.
        :rtype: list
        """
        self.logger.debug("Asset: '%s'", asset)
        self.asset_checker(asset)
        return self._api_query("get_available_data_types_for_asset", asset.lower())['result']

    def get_asset_data_for_time_range(self, asset, data_type, begin_timestamp, end_timestamp):
        """
        Returns daily values of specified data type for specified asset on
        dates that belong to closed interval between begin_timestamp and end_timestamp.

        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param data_type: Unique ID corresponding to the metric.
        :type data_type: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: List of lists, with unix timestamp and raw values.
        :rtype: list
        """
        self.logger.debug("Asset: '%s'", asset)
        self.logger.debug("Data Type: '%s'", data_type)
        self.logger.debug("Begin Timestamp: '%s'", begin_timestamp)
        self.logger.debug("End Timestamp: '%s'", end_timestamp)
        self.asset_checker(asset)
        self.data_type_checker(asset, data_type)
        self.timestamp_checker(begin_timestamp, end_timestamp)
        return self._api_query("get_asset_data_for_time_range", asset.lower(),
                               data_type, begin_timestamp, end_timestamp)['result']

    def get_active_addresses(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: Simple count (int) of addresses.
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_active_addresses' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset, "activeaddresses",
                                                  begin_timestamp, end_timestamp)

    #: An alias for :py:func:`get_active_addresses`
    [get_activeaddresses, getactiveaddresses, active_addresses,
     activeaddresses] = [get_active_addresses] * 4

    def get_adjusted_tx_volume(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: Volume (USD) of asset.
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_adjusted_tx_volume' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset,
                                                  "adjustedtxvolume",
                                                  begin_timestamp,
                                                  end_timestamp)

    #: An alias for :py:func:`get_adjusted_tx_volume`
    [get_adjustedtxvolume, getadjustedtxvolume, adjusted_tx_volume,
     adjustedtxvolume] = [get_adjusted_tx_volume] * 4

    def get_average_difficulty(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: A large int representing difficulty.
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_average_difficulty' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset, "averagedifficulty",
                                                  begin_timestamp, end_timestamp)

    #: An alias for :py:func:`get_average_difficulty`
    [get_averagedifficulty, getaveragedifficulty, average_difficulty,
     averagedifficulty] = [get_average_difficulty] * 4

    def get_block_count(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: Simple count (int) of blocks
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_block_count' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset, "blockcount",
                                                  begin_timestamp, end_timestamp)

    #: An alias for :py:func:`get_block_count`
    get_blockcount, getblockcount, block_count, blockcount = [get_block_count] * 4

    def get_block_size(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: Size of blocks in MB.
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_block_size' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset, "blocksize",
                                                  begin_timestamp, end_timestamp)

    #: An alias for :py:func:`get_block_size`
    get_blocksize, getblocksize, block_size, blocksize = [get_block_size] * 4

    def get_exchange_volume(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: Volume (int) of asset.
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_exchange_volume' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset, "exchangevolume",
                                                  begin_timestamp, end_timestamp)

    #: An alias for :py:func:`get_exchange_volume`
    [get_exchangevolume, getexchangevolume, exchange_volume,
     exchangevolume] = [get_exchange_volume] * 4

    def get_fees(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: Amount (int) of asset.
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_fees' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset, "fees",
                                                  begin_timestamp, end_timestamp)

    #: An alias for :py:func:`get_fees`
    getfees, fees = [get_fees] * 2

    def get_generated_coins(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: Amount (int) of asset.
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_generated_coins' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset, "generatedcoins",
                                                  begin_timestamp, end_timestamp)

    #: An alias for :py:func:`get_generated_coins`
    [get_generatedcoins, getgeneratedcoins, generated_coins,
     generatedcoins] = [get_generated_coins] * 4

    def get_market_cap(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: Amount (USD) of asset.
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_market_cap' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset, "marketcap",
                                                  begin_timestamp, end_timestamp)

    #: An alias for :py:func:`get_market_cap`
    get_marketcap, getmarketcap, market_cap, marketcap = [get_market_cap] * 4

    def get_median_fee(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: Amount (int) of asset.
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_median_fee' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset, "medianfee",
                                                  begin_timestamp, end_timestamp)

    #: An alias for :py:func:`get_median_fee`
    get_medianfee, getmedianfee, median_fee, medianfee = [get_median_fee] * 4

    def get_median_tx_value(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: Amount (USD) of asset.
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_median_tx_value' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset, "mediantxvalue",
                                                  begin_timestamp, end_timestamp)

    #: An alias for :py:func:`get_median_tx_value`
    get_mediantxvalue, getmediantxvalue, median_tx_value, mediantxvalue = [get_median_tx_value] * 4

    def get_payment_count(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: Amount (int) of asset.
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_payment_count' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset, "paymentcount",
                                                  begin_timestamp, end_timestamp)

    #: An alias for :py:func:`get_payment_count`
    get_paymentcount, getpaymentcount, payment_count, paymentcount = [get_payment_count] * 4

    def get_price(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: Value (float) in USD.
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_price' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset, "price",
                                                  begin_timestamp, end_timestamp)

    #: An alias for :py:func:`get_price`
    getprice, price = [get_price] * 2

    def get_realized_cap(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: Value (float) in USD
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_realized_cap' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset, "realizedcap",
                                                  begin_timestamp, end_timestamp)

    #: An alias for :py:func:`get_realized_cap`
    get_realizedcap, getrealizedcap, realized_cap, realizedcap = [get_realized_cap] * 4

    def get_tx_count(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: Simple count (int).
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_tx_count' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset, "txcount",
                                                  begin_timestamp, end_timestamp)

    #: An alias for :py:func:`get_tx_count`
    get_txcount, gettxcount, tx_count, txcount = [get_tx_count] * 4

    def get_tx_volume(self, asset, begin_timestamp, end_timestamp):
        """
        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int

        :return: Volume (int) of asset.
        :rtype: list
        """
        self.logger.debug("Redirecting 'get_tx_volume' to 'get_asset_data_for_time_range'")
        return self.get_asset_data_for_time_range(asset, "txcount",
                                                  begin_timestamp, end_timestamp)

    #: An alias for :py:func:`get_tx_volume`
    get_txvolume, gettxvolume, tx_volume, txcount = [get_tx_volume] * 4

    def asset_checker(self, asset):
        """
        Helper function to determine if the requested asset is valid or not.

        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str
        """
        if asset in self.get_supported_assets():
            pass
        else:
            self.logger.debug("Asset: '%s'", asset)
            raise InvalidAssetError

    def data_type_checker(self, asset, data_type):
        """
        Helper function to determine if the requested data type exists for a given asset

        :param asset: Unique ID corresponding to the asset's ticker: btc, ltc, gno
        :type asset: str

        :param data_type: Unique ID corresponding to the metric.
        :type data_type: str
        """
        if data_type in self.get_available_data_types_for_asset(asset):
            pass
        else:
            self.logger.debug("Asset: '%s'", asset)
            self.logger.debug("Data Type: '%s'", data_type)
            raise InvalidDataTypeError

    def timestamp_checker(self, begin_timestamp, end_timestamp):
        """
        Helper function to determine if the provided timerange is valid.

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or int

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or int
        """
        if begin_timestamp <= end_timestamp:
            pass
        else:
            self.logger.debug("Begin Timestamp: '%s'", begin_timestamp)
            self.logger.debug("End Timestamp: '%s'", end_timestamp)
            raise InvalidTimeRangeError



"""
Coin Metrics API Community Module Definitions
"""

import logging
from .base import Base
from .errors import InvalidAssetError, InvalidMetricError

class Community(Base):
    """
    Coin Metrics API Community Object
    """

    # Due to the conveniance methods, we trigger R0904: too-many-public-methods
    # even though this is desired.
    # pylint: disable=R0904

    def __init__(self, api_key=""):
        """
        Initialize the Community API exactly the same way as the same way as
        :py:func:`coinmetrics.base.Base.__init__`. An optional :samp:`api_key` can be supplied.

        :param api_key: API key to be used for the Pro API.
        :type api_key: str, optional
        """
        super().__init__()
        self.logger = logging.getLogger(__name__)

    def get_asset_info(self, assets=""):
        """
        Fetch asset(s) information. Including exchanges the asset is in,
        friendly name, markets, metrics, and most recent and oldest data
        timestamps. No specified asset will return all available asset info.

        :param assets: Unique ID corresponding to the asset's ticker.
        :type assets: str, optional

        :return: Asset information.
        :rtype: list of dict
        """
        self.logger.debug("Assets: '%s'", assets)
        if assets != "":
            options = {"subset": assets}
            self.asset_checker(assets)
        else:
            options = {}
        return self._api_query("asset_info", options)['assetsInfo']

    def get_exchange_info(self, exchanges=""):
        """
        Fetch exchange(s) information. Including assets, quotes, markets,
        and most recent and oldest data timestamps for each. No specified
        exchange will return all available exchange info.

        :param exchanges: Unique ID corresponding to the exchange.
        :type exchanges: str, optional

        :return: Exchange information.
        :rtype: list of dict
        """
        self.logger.debug("Exchanges: '%s'", exchanges)
        if exchanges != "":
            options = {"subset": exchanges}
            self.exchange_checker(exchanges)
        else:
            options = {}
        return self._api_query("exchange_info", options)['exchangesInfo']

    def get_metric_info(self, metrics=""):
        """
        Fetch metric(s) information. Including friendly name, dscription,
        category.

        :param metrics: Unique ID corresponding to the metrics.
        :type metrics: str, optional

        :return: Metric information.
        :rtype: list of dict
        """
        self.logger.debug("Metrics: '%s'", metrics)
        if metrics != "":
            self.metric_checker(metrics)
            options = {"subset": metrics}
        else:
            options = {}
        return self._api_query("metric_info", options)['metricsInfo']

    def get_market_info(self, markets=""):
        """
        Fetch market(s) information. Includes assets, quotes,
        and most recent and oldest data timestamps for each.

        :param markets: Unique ID corresponding to the market.
        :type markets: str, optional

        :return: Market information.
        :rtype: list of dict
        """
        self.logger.debug("Markets: '%s'", markets)
        if markets != "":
            self.market_checker(markets)
            options = {"subset": markets}
        else:
            options = {}
        return self._api_query("market_info", options)['marketsInfo']

    def get_asset_metrics(self, asset):
        """
        Fetch list of available metrics for a given asset. This will only
        fetch metrics corresponding to one asset at a time.

        :param asset: Unique ID corresponding to the asset's ticker.
        :type asset: str

        :return: List of supported metrics for the asset.
        :rtype: list
        """
        self.logger.debug("Asset: '%s'", asset)
        if "," in asset:
            raise InvalidAssetError("""Can only fetch compatible metrics from
                                       a single asset at a time.""")
        return self.get_asset_info(asset)[0]['metrics']

    #: An alias for :py:func:`get_asset_metrics` (backwards compatibility)
    get_available_data_types_for_asset = get_asset_metrics

    def asset_metric_checker(self, asset, metrics):
        """
        Helper function to determine if the requested metric(s) is(are) valid
        for an asset. This will only validate metrics corresponding to one
        asset at a time.

        :param asset: Unique ID corresponding to the asset's ticker.
        :type asset: str

        :param metric: Unique ID corresponding to metric.
        :type metric: str
        """
        self.logger.debug("Asset: '%s'", asset)
        self.logger.debug("Metrics: '%s'", metrics)
        metrics = metrics.split(",")
        reference = self.get_asset_metrics(asset)
        for metric in metrics:
            if metric in reference:
                pass
            else:
                raise InvalidMetricError("""Invalid metric '{}' for the given
                                            asset '{}'.""".format(metric, asset))

    def get_asset_metric_data(self, asset, metrics, start, end, time_agg="day"):
        """
        Fetch metric(s) data given a specified asset, and timeframe.
        See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :param asset: Unique ID corresponding to the asset's ticker.
        :type asset: str

        :param metrics: Unique ID corresponding to the metrics.
        :type metrics: str

        :param begin_timestamp: Start of time inverval.
        :type begin_timestamp: str or datetime

        :param end_timestamp: End of time inverval.
        :type end_timestamp: str or datetime

        :param time_agg: Interval the time is descritized into: day, hour.
        :type time_agg: str

        :return: Coin Metrics API data object. See the `API reference`_ for details
        :rtype: dict

        .. _`API reference`: https://docs.coinmetrics.io/api/v2/#operation/getAssetMetricsData
        """
        #pylint: disable=R0913
        if metrics == "all" or None:
            metrics = ','.join(self.get_asset_metrics(asset))
        self.logger.debug("asset: '%s'", asset)
        self.logger.debug("Metrics: '%s'", metrics)
        self.logger.debug("Start Timestamp: '%s'", start)
        self.logger.debug("End Timestamp: '%s'", end)
        self.asset_checker(asset)
        self.metric_checker(metrics)
        self.timestamp_checker(start, end)
        endpoint = "assets/%s/metricdata" % asset
        options = {"metrics": metrics, "start": start, "end": end, "time_agg": time_agg}
        return self._api_query(endpoint, options)['metricData']

    #: An alias for :py:func:`get_asset_metric_data` (backwards compatibility)
    get_asset_data_for_time_range = get_asset_metric_data

    def get_active_addresses(self, assets, start, end):
        """
        The sum count of unique addresses that were active in the network
        (either as a recipient or originator of a ledger change) that day.
        All parties in a ledger change action (recipients and originators)
        are counted. Individual addresses are not double-counted if
        previously active. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "AdrActCnt", start, end)

    #: An alias for :py:func:`get_active_addresses`
    active_addresses, activeaddresses, AdrActCnt = [get_active_addresses] * 3

    def get_block_count(self, assets, start, end):
        """
        The sum count of unique addresses that were active in the network
        (either as a recipient or originator of a ledger change) that day.
        All parties in a ledger change action (recipients and originators)
        are counted. Individual addresses are not double-counted if
        previously active. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "BlkCnt", start, end)

    #: An alias for :py:func:`get_block_count`
    block_count, blockcount, BlkCnt = [get_block_count] * 3

    def get_mean_block_size(self, assets, start, end):
        """
        The mean size (in bytes) of all blocks created that day.
        See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "BlkSizeMeanByte", start, end)

    #: An alias for :py:func:`get_mean_block_size`
    get_meanblocksize, meanblocksize, BlkSizeMeanByte = [get_mean_block_size] * 3

    def get_mvrv_cur(self, assets, start, end):
        """
        The ratio of the sum USD value of the current supply to the sum
        "realized" USD value of the current supply. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "CapMVRVCur", start, end)

    #: An alias for :py:func:`get_mvrv_cur`
    get_mvrvcur, mvrvcur, CapMVRVCur = [get_mvrv_cur] * 3

    def get_real_cap(self, assets, start, end):
        """
        The sum USD value based on the USD closing price on the day
        that a native unit last moved (i.e., last transacted)
        for all native units. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "CapRealUSD", start, end)

    #: An alias for :py:func:`get_real_cap`
    get_realcap, real_cap, CapRealUSD = [get_real_cap] * 3

    def get_diff_mean(self, assets, start, end):
        """
        The mean difficulty of finding a hash that meets the protocol
        designated requirement (i.e., the difficulty of finding a
        new block) that day. The requirement is unique to each
        applicable cryptocurrency protocol. Difficulty is adjusted
        periodically by the protocol as a function of how much
        hashing power is being deployed by miners.
        See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "DiffMean", start, end)

    #: An alias for :py:func:`get_diff_mean`
    get_diffmean, diff_mean, DiffMean = [get_diff_mean] * 3

    def get_fee_mean(self, assets, start, end):
        """
        The USD value of the mean fee per transaction that day.
        See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "FeeMeanUSD", start, end)

    #: An alias for :py:func:`get_fee_mean`
    get_feemean, fee_mean, FeeMeanUSD = [get_fee_mean] * 3

    def get_median_fee(self, assets, start, end):
        """
        The USD value of the median fee per transaction that day.
        See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "FeeMedUSD", start, end)

    #: An alias for :py:func:`get_median_fee`
    get_medianfee, median_fee, FeeMedUSD = [get_median_fee] * 3

    def get_fee_total(self, assets, start, end):
        """
        The sum USD value of all fees paid to miners that day. Fees do not
        include new issuance. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "FeeTotUSD", start, end)

    #: An alias for :py:func:`get_fee_total`
    get_feetotal, fee_total, FeeTotUSD = [get_fee_total] * 3

    def get_units_issued(self, assets, start, end):
        """
        The sum of new native units issued that day. Only those
        native units that are issued by a protocol-mandated
        continuous emission schedule are included.
        See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "IssContNtv", start, end)

    #: An alias for :py:func:`get_issued`
    get_unitsissued, units_issued, IssContNtv = [get_units_issued] * 3

    def get_units_issued_ann_pct(self, assets, start, end):
        """
        The percentage of new native units (continuous) issued on
        that day, extrapolated to one year (i.e., multiplied by 365),
        and divided by the current supply on that day. Also referred
        to as the annual inflation rate. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "IssContPctAnn", start, end)

    #: An alias for :py:func:`get_units_issued_ann_pct`
    get_unitsannpct, units_ann_pct, IssContPctAnn = [get_units_issued_ann_pct] * 3

    def get_units_issued_usd(self, assets, start, end):
        """
        The sum USD value of all new native units issued that day.
        See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "IssTotUSD", start, end)

    #: An alias for :py:func:`get_units_issued_usd`
    get_unitsissuedusd, units_issued_usd, IssTotUSD = [get_units_issued_usd] * 3

    def get_nvt_adj(self, assets, start, end):
        """
        The ratio of the network value (or market capitalization,
        current supply) divided by the adjusted transfer value.
        Also referred to as NVT. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "NVTAdj", start, end)

    #: An alias for :py:func:`get_nvt_adj`
    get_nvtadj, nvt_adj, NVTAdj = [get_nvt_adj] * 3

    def get_nvt_adj_90(self, assets, start, end):
        """
        The ratio of the network value (or market capitalization,
        current supply) to the 90-day moving average of the adjusted
        transfer value. Also referred to as NVT. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "NVTAdj90", start, end)

    #: An alias for :py:func:`get_nvt_adj_90`
    get_nvtadj90, nvt_adj_90, NVTAdj90 = [get_nvt_adj_90] * 3

    def get_price_btc(self, assets, start, end):
        """
        The fixed closing price of the asset as of 00:00 UTC the
        following day (i.e., midnight UTC of the current day)
        denominated in USD. This price is generated by Coin Metrics'
        fixing/reference rate service. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "PriceBTC", start, end)

    #: An alias for :py:func:`get_price_btc`
    get_pricebtc, price_btc, NVTAdj90 = [get_price_btc] * 3

    def get_price_usd(self, assets, start, end):
        """
        The fixed closing price of the asset as of 00:00 UTC the
        following day (i.e., midnight UTC of the current day)
        denominated in BTC. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "PriceUSD", start, end)

    #: An alias for :py:func:`get_price_usd`
    get_priceusd, price_usd, PriceUSD = [get_price_usd] * 3

    def get_cur_sply(self, assets, start, end):
        """
        The sum of all native units ever created and visible on the
        ledger (i.e., issued) as of that day. For account-based
        protocols, only accounts with positive balances are counted.
        See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """

        return self.get_asset_metric_data(assets, "SplyCur", start, end)

    #: An alias for :py:func:`get_cur_sply`
    get_cursply, cur_sply, PriceUSD = [get_cur_sply] * 3

    def get_tx_count(self, assets, start, end):
        """
        The sum count of transactions that day. Transactions
        represent a bundle of intended actions to alter the ledger
        initiated by a user (human or machine). See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """

        return self.get_asset_metric_data(assets, "TxCnt", start, end)

    #: An alias for :py:func:`get_tx_count`
    get_txcount, tx_count, TxCnt = [get_tx_count] * 3

    def get_txtfr_count(self, assets, start, end):
        """
        The sum count of transfers that day. Transfers represent
        movements of native units from one ledger entity to another
        distinct ledger entity. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "TxTfr", start, end)

    #: An alias for :py:func:`get_txtfr_count`
    get_txtfrcount, txtfr_count, TxTfr = [get_txtfr_count] * 3

    def get_txtfr_val_adj(self, assets, start, end):
        """
        The sum of native units transferred that day removing noise and
        certain artifacts. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "TxTfrValAdjNtv", start, end)

    #: An alias for :py:func:`get_txtfr_val_adj`
    get_txtfrvaladj, txtfr_val_adj, TxTfrValAdjNtv = [get_txtfr_val_adj] * 3

    def get_txtfr_val_adj_usd(self, assets, start, end):
        """
        The USD value of the sum of native units transferred that day removing
        noise and certain artifacts. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "TxTfrValAdjUSD", start, end)

    #: An alias for :py:func:`get_txtfr_val_adj_usd`
    get_txtfrvaladj, txtfr_val_adj, TxTfrValAdjUSD = [get_txtfr_val_adj] * 3

    def get_txtfr_val_mean(self, assets, start, end):
        """
        The mean count of native units transferred per transaction
        (i.e., the mean "size" of a transaction) that day.
        See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "TxTfrValMeanNtv", start, end)

    #: An alias for :py:func:`get_txtfr_val_mean`
    get_txtfrvalmean, txtfr_val_mean, TxTfrValMeanNtv = [get_txtfr_val_mean] * 3

    def get_txtfr_val_mean_usd(self, assets, start, end):
        """
        The sum USD value of native units transferred divided by
        the count of transfers (i.e., the mean "size" in USD of a
        transfer) that day. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "TxTfrValMeanUSD", start, end)

    #: An alias for :py:func:`get_txtfr_val_mean_usd`
    get_txtfrvalmeanusd, txtfr_val_mean_usd, TxTfrValMeanUSD = [get_txtfr_val_mean_usd] * 3

    def get_txtfr_val_med(self, assets, start, end):
        """
        The median count of native units transferred per transfer
        (i.e., the median "size" of a transfer) that day.
        See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "TxTfrValMedNtv", start, end)

    #: An alias for :py:func:`get_txtfr_val_med`
    get_txtfrvalmed, txtfr_val_med, TxTfrValMedNtv = [get_txtfr_val_med] * 3

    def get_txtfr_val_med_usd(self, assets, start, end):
        """
        The median USD value transferred per transfer (i.e., the
        median "size" in USD of a transfer) that day.
        See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "TxTfrValMedUSD", start, end)

    #: An alias for :py:func:`get_txtfr_val_med_usd`
    get_txtfrvalmedusd, txtfr_val_med_usd, TxTfrValMedUSD = [get_txtfr_val_med_usd] * 3

    def get_txtfr_val(self, assets, start, end):
        """
        The sum of native units transferred (i.e., the aggregate
        "size" of all transfers) that day. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "TxTfrValNtv", start, end)

    #: An alias for :py:func:`get_txtfr_val`
    get_txtfrval, txtfr_val, TxTfrValNtv = [get_txtfr_val] * 3

    def get_txtfr_val_usd(self, assets, start, end):
        """
        The sum USD value of all native units transferred
        (i.e., the aggregate size in USD of all transfers)
        that day. See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "TxTfrValUSD", start, end)

    #: An alias for :py:func:`get_txtfr_val_usd`
    get_txtfrvalusd, txtfr_val_usd, TxTfrValUSD = [get_txtfr_val_usd] * 3

    def get_vty_ret_180d(self, assets, start, end):
        """
        The 180D volatility, measured as the deviation of
        log returns See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "VtyDayRet180d", start, end)

    #: An alias for :py:func:`get_vty_ret_180d`
    get_vtyret180d, vty_ret_180d, VtyDayRet180d = [get_vty_ret_180d] * 3

    def get_vty_ret_30d(self, assets, start, end):
        """
        The 30D volatility, measured as the deviation of log returns See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "VtyDayRet30d", start, end)

    #: An alias for :py:func:`get_vty_ret_30d`
    get_vtyret30d, vty_ret_30d, VtyDayRet30d = [get_vty_ret_30d] * 3

    def get_vty_ret_60d(self, assets, start, end):
        """
        The 60D volatility, measured as the deviation of log returns.
        See: `Data Dictionary`_.

        .. _`Data Dictionary`: https://coinmetrics.io/community-data-dictionary/

        :Parameters: See see :py:func:`get_asset_metric_data` for parameter and return details.
        """
        return self.get_asset_metric_data(assets, "VtyDayRet60d", start, end)

    #: An alias for :py:func:`get_vty_ret_60d`
    get_vtyret60d, vty_ret_60d, VtyDayRet60d = [get_vty_ret_60d] * 3



"""
Coin Metrics API Conversion Utilities

The functions below are not internally used by the :samp:`coinmetrics` library. They are
transformations to allow data that was fetched to be more easily processed by another
process.

Usage Examples: :ref:`usage`
"""

def cm_to_pandas(data):
    """
    Convert an object output from :py:func:`coinmetrics.community.Community.get_asset_metric_data`
    to a Pandas object for further processing.

    :param object: Raw data object to convert to Pandas datagram.
    :type object: dict

    :return: Pandas dataframe form of original object.
    :rtype: pandas dataframe
    """
    import pandas as pd
    if not isinstance(data, pd.DataFrame):
        raw_index = [row['time'] for row in data['series']]
        raw_values = [row['values'] for row in data['series']]
        pandas_dataframe = pd.DataFrame(index=raw_index,
                                        data=raw_values,
                                        columns=data['metrics'])
        pandas_dataframe = pandas_dataframe.astype(float)
        return pandas_dataframe
    return data

def normalize(data):
    """
    Convert an object output from :py:func:`coinmetrics.community.Community.get_asset_metric_data`
    to a standard list of dictionaries for further processing.

    :param object: Raw data object to convert to list of dict.
    :type object: dict

    :return: A normalized list of dictionaries
    :rtype: list
    """
    return [{**{'time': row['time']}, **dict(zip(data['metrics'], row['values']))}
            for row in data['series']]

def csv(data, path):
    """
    Convert an object output from :py:func:`coinmetrics.community.Community.get_asset_metric_data`
    to a standard list of dictionaries for further processing.

    :param object: Raw data object to convert to Pandas datagram.
    :type object: dict

    :param path: Location to save the CSV file to.
    :type path: str, optional
    """
    data = cm_to_pandas(data)
    data.to_csv(path_or_buf=path)



#!/usr/bin/env python3
"""
Setup for Coin Metrics API.
"""
import os
from setuptools import setup

with open('requirements.txt') as f:
    required = f.read().splitlines()

setup(
	name='coinmetrics',
	version='0.2.3',
	author="Robert Rice",
	author_email="h4110w33n@gmail.com",
	url='https://github.com/h4110w33n/coinmetrics',
	description='An unofficial library to standardize queries to the Coin Metrics (coinmetrics.io) API.',
	long_description='An unofficial library to standardize queries to the Coin Metrics (coinmetrics.io) API.',
	keywords=['coinmetrics', 'crypto', 'cryptocurrency', 'rest', 'api'],
	classifiers=[
		'Development Status :: 5 - Production/Stable',
		'Intended Audience :: Developers',
		'Intended Audience :: Education',
		'Intended Audience :: Financial and Insurance Industry',
		'Intended Audience :: Information Technology',
		'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
		'Operating System :: OS Independent',
		'Programming Language :: Python :: 3.3',
		'Programming Language :: Python :: 3.4',
		'Programming Language :: Python :: 3.5',
		'Programming Language :: Python :: 3.6',
		'Programming Language :: Python :: 3.7',
		'Programming Language :: Python',
		'Topic :: Internet :: WWW/HTTP :: Indexing/Search',
		'Topic :: Scientific/Engineering :: Information Analysis',
		'Topic :: Scientific/Engineering :: Mathematics',
		'Topic :: Software Development :: Libraries :: Python Modules',
	],
	install_requires=required,
	provides=['coinmetrics'],
	license='GNU General Public License v3 (GPLv3)',
	packages=['coinmetrics'],
)



"""
Coin Metrics API Module Includes
"""

from .base import Base
from .community import Community
from .legacy import CommunityLegacy
from .pro import Pro
from .utils import cm_to_pandas, normalize, csv



"""
Coin Metrics API Error Module Definitions
"""

# We do not call these errors from this file, so we should ignore the lint error for it here.
# pylint: disable=W0611

class Error(Exception):
    """
    Base class for other exceptions.
    """

class InvalidAssetError(Error):
    """
    Raise an error when the given asset doesn't exist.
    """

class InvalidTimeRangeError(Error):
    """
    Raise and error when the given time range is not-sane (i.e. end before start).
    """

class InvalidMetricError(Error):
    """
    Raise and error when the given metric doesn't exist for the specified asset.
    """

class InvalidExchangeError(Error):
    """
    Raise and error when the given metric doesn't exist for the specified asset.
    """

class InvalidMarketError(Error):
    """
    Raise and error when the given metric doesn't exist for the specified asset.
    """

class InvalidDataTypeError(Error):
    """
    Raise and error when the given data_type doesn't exist for the specified asset.
    """
